﻿;
; Chipmunk2D4PB - Demo Manager
;

;- The "Global" Chipmunk2D4PB structure
; Structure cpSprite
;   *spriteID  
;   *body.cpBody
;   *shape.cpShape
;   *pos.cpVect
;   angle.cpFloat
;   *data;
; EndStructure

#ArmLength1 = 100:
#ArmLength2 = 120

XIncludeFile ".\include\Chipmunk2D4PB.pbi"

Global wX.cpFloat = 600
Global wY.cpFloat = 600

CompilerIf Subsystem("DX9")
  MP_Graphics3DWindow(0,0, wX,wY, "ee", #PB_Window_ScreenCentered|#PB_Window_SystemMenu)
  MP_VSync(0) 
CompilerElse
  If InitSprite() = 0 Or InitKeyboard() = 0 Or InitMouse() = 0
    MessageRequester("Error", "Can't open the sprite system", 0)
    End
  EndIf
  OpenWindow(0, 0, 0, wX,wY, "ee", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
  OpenWindowedScreen(WindowID(0), 0,0, wX,wY, 0, 0, 0);, #PB_Screen_NoSynchronization )
CompilerEndIf


Global *mouseSprite.cpSprite = #Null
Global fontID = LoadFont(#PB_Any, "Small Fonts", 4)


;
;-  Debug Draw stuff
;

Global *DrawShape = GetDrawShapeAddress()
Global *mouseBody.cpBody = cpBodyNewKinematic()
SetMouseBodyAddress(*mouseBody)



; Wrapper globals
Global *player.cpSprite  = #Null
Global ChipmunkDemoMouse.cpVect
Global buttonFlag = #False
Global keyPressed


Procedure Body_SegmentAdd(*a.cpVect, *b.cpVect, radius.d, mass.i)
  *Segment.cpSprite = CreateSegmentElement(*space,  *a\x,  *a\y,  *b\x,  *b\y,  1, 1)
  ProcedureReturn *Segment
EndProcedure

Procedure Body_CircleAdd(x.d, y.d, radius.d, mass)
  *Circle.cpSprite = CreateCircleElement(*space, x, y, radius, 0)
  Image = CreateImage(#PB_Any, radius * 2, radius * 2, 24, #Black)
  StartDrawing(ImageOutput(Image))
  Circle(radius, radius, radius-1, RGB(Random(245,10), Random(245,10), Random(245,10)))
  Circle(radius, radius/2, 2, RGB(Random(245,10), Random(245,10), Random(245,10)))
  StopDrawing()
  
  CompilerIf Subsystem("DX9")
    Texture = MP_ImageToTexture(Image)  
    *Circle\spriteID  = MP_SpriteFromTexture(Texture) 
  CompilerElse
    *Circle\spriteID = CreateSprite(#PB_Any, radius * 2, radius * 2)
    StartDrawing(SpriteOutput(*Circle\spriteID))
    DrawImage(ImageID(Image),0,0)
    StopDrawing()
  CompilerEndIf
  ProcedureReturn *Circle
EndProcedure

Procedure Body_BoxAdd(x.d, y.d, width.d, height.d, mass)
  *Box.cpSprite = CreateBoxElement(*space, x, y, width, height, 0)
  
  cpShapeSetFriction(*Box\Shape, 0.25)
  cpShapeSetElasticity(*Box\Shape, 0.9)
  Image = CreateImage(#PB_Any, Width, Height, 24,  RGB(Random(255,10), Random(255,10), Random(255,10)))
  StartDrawing(ImageOutput(image))
  DrawingFont(FontID(fontID)) 
  
  DrawingMode(#PB_2DDrawing_Transparent)
  DrawText(Width * 0.5 - TextWidth("Chipmunk") * 0.5, Height * 0.5 - TextHeight("Chipmunk") * 0.5, "Chipmunk", RGB(0,0,0))
  StopDrawing()
  
  CompilerIf Subsystem("DX9")
    Texture = MP_ImageToTexture(Image)  
    *Box\spriteID  = MP_SpriteFromTexture(Texture) 
  CompilerElse
    *Box\spriteID = CreateSprite(#PB_Any, Width, Height)
    StartDrawing(SpriteOutput(*Box\spriteID))
    DrawImage(ImageID(Image),0,0)
    StopDrawing()
  CompilerEndIf
  ProcedureReturn *Box
EndProcedure



Procedure AddPlayer(x, y, width, height, mass)
  *player.cpSprite = CreateBoxElement(*space, x, y, width, height, 0)
  cpShapeSetFriction(*player\Shape, 1)
  
  Image = CreateImage(#PB_Any, Width, Height, 24,  #Red )
  CompilerIf Subsystem("DX9")
    Texture = MP_ImageToTexture(Image)  
    *player\spriteID  = MP_SpriteFromTexture(Texture) 
  CompilerElse   
    *player\spriteID = CreateSprite(#PB_Any, Width, Height)
    StartDrawing(SpriteOutput(*player\spriteID))
    DrawImage(ImageID(Image),0,0)
    StopDrawing()
  CompilerEndIf
  
EndProcedure


Global title.s = "Chipmunk2D4PB - From Scratch"

Structure ChipmunkDemo 
  title.s 
  timestep.d
  *initFunc
  *updateFunc
  *destroyFunc
  description.s ; few more info for the users ;-)
EndStructure


Global NewList ChipmunkDemo.ChipmunkDemo()

;- Default initFunc()
Procedure initFunc()
  *mouseSprite = AllocateStructure(cpSprite)
  ;create mouse sprite
  Image = CreateImage(#PB_Any, 8,8)
  StartDrawing(ImageOutput(Image))
  Circle(4,4,2,#Red)
  StopDrawing()
  
  CompilerIf Subsystem("DX9")
    Texture = MP_ImageToTexture(Image)  
    *mouseSprite\spriteID  = MP_SpriteFromTexture(Texture) 
  CompilerElse
    *mouseSprite\spriteID = CreateSprite(#PB_Any,8,8)
    StartDrawing(SpriteOutput(*mouseSprite\spriteID))
    DrawImage(ImageID(image), 0,0)
    StopDrawing()
  CompilerEndIf
  
  *space = cpSpaceNew()
  *staticBody = cpSpaceGetStaticBody(*space);
  SetSpaceAddress(*space)
  cpSpaceSetIterations(*space, 60)  
  
  cpSpaceSetGravity(*space, cpv(_A,0, 200))
     cpSpaceSetSleepTimeThreshold(*space, 0.5)
     cpSpaceSetCollisionSlop(*space, 0.5)    
EndProcedure

;- Default updateFunc()
Procedure updateFunc()
  cpSpaceStep(*space, ChipmunkDemo()\timestep)
EndProcedure

;- Default destroyFunc()
Procedure destroyFunc()
  ChipmunkDemoFreeSpaceChildren(*space)
  cpSpaceFree(*space)
  FreeImage(#PB_All)
  FreeSprite(#PB_All)
EndProcedure


XIncludeFile".\Demos\LogoSmash.pbi"
XIncludeFile".\Demos\PyramidTopple.pbi"
XIncludeFile".\Demos\PyramidStack.pbi"
XIncludeFile".\Demos\Plink.pbi"
XIncludeFile".\Demos\Tumble.pbi"
XIncludeFile".\Demos\Tank.pbi"
XIncludeFile".\Demos\OneWay.pbi"
XIncludeFile".\Demos\Chains.pbi"
;
XIncludeFile".\Demos\DoublePendulum.pbi"
XIncludeFile".\Demos\GrainsDeSable.pbi"
XIncludeFile".\Demos\GravityBounce.pbi"

; XIncludeFile".\Demos\moon_buggy.pbi"
; XIncludeFile".\Demos\Ropes.pbi"



;- Game loop ------------------
FirstElement(ChipmunkDemo())

SetWindowTitle(0, ChipmunkDemo()\title) 
CallThisFunction(ChipmunkDemo()\initFunc)

ReleaseMouseFlag = 0
Repeat
  CompilerIf Subsystem("DX9")
  CompilerElse
    ReleaseMouse(ReleaseMouseFlag)
  CompilerEndIf
  
  event = 1
  While event
    event = WindowEvent()
    If event = #PB_Event_CloseWindow
      Quit = 1
    EndIf
    
  Wend
  
  ;Mouse actions
  xold = ChipmunkDemoMouse\x
  yold = ChipmunkDemoMouse\y
  
  CompilerIf Subsystem("DX9")
    
    ChipmunkDemoMouse\x = WindowMouseX(0)
    ChipmunkDemoMouse\y = WindowMouseY(0)
    
    mButton_Left = MP_MouseButtonDown(0)  
    mButton_Right = MP_MouseButtonDown(1)
    mButton_Middle = MP_MouseButtonDown(2)
    
  CompilerElse
    ExamineMouse()   
    ChipmunkDemoMouse\x = MouseX()
    ChipmunkDemoMouse\y = MouseY()
    
    mButton_Left = MouseButton(#PB_MouseButton_Left)  
    mButton_Right = MouseButton(#PB_MouseButton_Right )
    mButton_Middle = MouseButton(#PB_MouseButton_Middle)
  CompilerEndIf
  
  If ChipmunkDemoMouse\x< 0 : ChipmunkDemoMouse\x = xold: EndIf
  If ChipmunkDemoMouse\y< 0 : ChipmunkDemoMouse\y = yold: EndIf
  
  If mButton_Right <> 0
    buttonFlag = #False
    cpBodySetPosition(*mouseBody, cpv(_A, ChipmunkDemoMouse\x, ChipmunkDemoMouse\y))
    Click(*space, 1, 0, ChipmunkDemoMouse\x, ChipmunkDemoMouse\y) 
  EndIf
  If mButton_Left <> 0
    cpBodySetPosition(*mouseBody, cpv(_A, ChipmunkDemoMouse\x, ChipmunkDemoMouse\y))
    If buttonFlag = #False       
      Click(*space, 1, 1,  ChipmunkDemoMouse\x,  ChipmunkDemoMouse\y)
      buttonFlag = #True
    EndIf
  EndIf
  
  
  
  CompilerIf Subsystem("DX9")
    If MP_KeyDown(#PB_Key_Escape)
      Quit = 1
    EndIf
    ; do the sprite & screen management at every frame
    If MP_KeyDown(#PB_Key_Up)    And playerY > 0   : playerY -3 : EndIf  
    If MP_KeyDown(#PB_Key_Down)  And playerY < 280 : playerY +3 : EndIf  
    If MP_KeyDown(#PB_Key_Left)  And playerX > 0   : playerX -3 : EndIf  
    If MP_KeyDown(#PB_Key_Right) And playerX < 300 : playerX +3 : EndIf  
    If MP_KeyDown(#PB_Key_F12) :PressedF12 = #True: EndIf
    If MP_KeyDown(#PB_Key_D) 
      Chipmunk4PBDebugFlag ! 1:  
      SetChipmunk4PBDebugFlag(Chipmunk4PBDebugFlag)  
    EndIf
    If MP_KeyDown(#PB_Key_F1)
      ReleaseMouseFlag ! 1
      ;  ReleaseMouse(ReleaseMouseFlag)
      InputReleased = 1
    EndIf
    
  CompilerElse
    ExamineKeyboard()
    
    If KeyboardPushed(#PB_Key_Escape)
      Quit = 1
    EndIf
     If KeyboardPushed(#PB_Key_Pad0): keyPressed = 0: EndIf
     If KeyboardPushed(#PB_Key_Pad1): keyPressed = 1: EndIf
     If KeyboardPushed(#PB_Key_Pad2): keyPressed = 2: EndIf
     If KeyboardPushed(#PB_Key_Pad3): keyPressed = 3: EndIf
     If KeyboardPushed(#PB_Key_Pad4): keyPressed = 4: EndIf
     If KeyboardPushed(#PB_Key_Pad5): keyPressed = 5: EndIf
     If KeyboardPushed(#PB_Key_Pad6): keyPressed = 6: EndIf
     If KeyboardPushed(#PB_Key_Pad7): keyPressed = 7: EndIf
     If KeyboardPushed(#PB_Key_Pad8): keyPressed = 8: EndIf
     If KeyboardPushed(#PB_Key_Pad9): keyPressed = 9: EndIf
    
    ; do the sprite & screen management at every frame
    If KeyboardPushed(#PB_Key_Up)    And playerY > 0   : playerY -3 : EndIf  
    If KeyboardPushed(#PB_Key_Down)  And playerY < 280 : playerY +3 : EndIf  
    If KeyboardPushed(#PB_Key_Left)  And playerX > 0   : playerX -3 : EndIf  
    If KeyboardPushed(#PB_Key_Right) And playerX < 300 : playerX +3 : EndIf  
    If KeyboardPushed(#PB_Key_F12) And PressedF12 = #False
      CallThisFunction(ChipmunkDemo()\destroyFunc)     
      res = NextElement(ChipmunkDemo())  
      If res = 0 
        ResetList(ChipmunkDemo())
        NextElement(ChipmunkDemo())
      EndIf
      SetWindowTitle(0, ChipmunkDemo()\title) 
      CallThisFunction(ChipmunkDemo()\initFunc)
      PressedF12 = #True    
    ElseIf KeyboardReleased(#PB_Key_F12)  
      PressedF12 = #False
    EndIf
    
    If KeyboardPushed(#PB_Key_D) 
      Chipmunk4PBDebugFlag ! 1:  
      SetChipmunk4PBDebugFlag(Chipmunk4PBDebugFlag)  
    EndIf
    If KeyboardPushed(#PB_Key_F1)
      ReleaseMouseFlag ! 1
      ReleaseMouse(ReleaseMouseFlag)
      InputReleased = 1
    EndIf
  CompilerEndIf
  
  If buttonFlag = #True ; Object attached 
    cpBodySetPosition(*mouseBody, cpv(_A, ChipmunkDemoMouse\x, ChipmunkDemoMouse\y))
  EndIf
  
  CompilerIf Subsystem("DX9")
    CallThisFunction(ChipmunkDemo()\updateFunc)
    MP_DrawText(10,10,"FPS: " + Str(MP_FPS()),font1,$ffff0000,0,0)  
    MP_DrawText(10,25, ChipmunkDemo()\description,font1,$ffff0000,0,0)
    cpSpaceEachShape(*space, *DrawShape, 1) 
    cpSpaceEachShape(*space, *DrawShape, 0)  
    
    MP_DrawSprite(*mouseSprite\spriteID, ChipmunkDemoMouse\x, ChipmunkDemoMouse\y)
    
    MP_RenderWorld()
    MP_Flip()
  CompilerElse
    ; FPS -------------------------------------  
    If fpsstart = 0  
      fpsstart = ElapsedMilliseconds()  
    EndIf  
    fpszaehler + 1  
    If ElapsedMilliseconds() - fpsstart >= 1000  
      fps = fpszaehler  
      fpszaehler = 0  
      fpsstart = ElapsedMilliseconds()  
    EndIf  
    
    ClearScreen(#Black)
    CallThisFunction(ChipmunkDemo()\updateFunc)
    
    StartDrawing(ScreenOutput())
    DrawText(10, 10, "FPS:" + Str(fps) )
    DrawText(10, 25, ChipmunkDemo()\description)
    cpSpaceEachShape(*space, *DrawShape, 0)     
    StopDrawing()
    cpSpaceEachShape(*space, *DrawShape, 1) 
    
    DisplayTransparentSprite(*mouseSprite\spriteID, ChipmunkDemoMouse\x, ChipmunkDemoMouse\y)
    
    FlipBuffers()
  CompilerEndIf
  
  
Until Quit = 1
CompilerIf Subsystem("DX9")
CompilerElse
  ReleaseMouse(1)
CompilerEndIf
CallThisFunction(ChipmunkDemo()\destroyFunc)
End
; IDE Options = PureBasic 5.70 LTS beta 1 (Windows - x86)
; ExecutableFormat = Console
; CursorPosition = 166
; FirstLine = 148
; Folding = ---
; Executable = ..\..\..\Chipmunk2D_wrapper_Demo_20180725.exe
; DisableDebugger