;
; Version 201807
;

;XIncludeFile "..\Residents\Chipmunk4PB_Resident.pbi"




ImportC "Chipmunk2D4PB.lib"
  GRAB_FILTER.cpShapeFilter As "?GRAB_FILTER@@3UcpShapeFilter@@A"; (struct cpShapeFilter GRAB_FILTER)
  NOT_GRABBABLE_FILTER.cpShapeFilter As "?NOT_GRABBABLE_FILTER@@3UcpShapeFilter@@A"; (struct cpShapeFilter NOT_GRABBABLE_FILTER)
  cpvzero.cpVect As "_cpvzero"
  cpTransformIdentity.cpTransform As "_cpTransformIdentity"
  CP_SHAPE_FILTER_NONE.cpShapeFilter As "_CP_SHAPE_FILTER_NONE"
  CP_SHAPE_FILTER_ALL.cpShapeFilter As "_CP_SHAPE_FILTER_ALL"
EndImport

;- Globals
;- cpVect variabels for functions
Global _A.cpVect
Global _B.cpVect
Global _C.cpVect
Global _D.cpVect
Global _E.cpVect
Global _F.cpVect
Global _G.cpVect
Global _H.cpVect
Global _I.cpVect

; Chipmunk2D globals
Global *DrawShape = GetDrawShapeAddress()

Global *space.cpSpace = #Null
Global *staticBody.cpBody = #Null




;- Procedures

ProcedureC BreakablejointPostStepRemove(*space.cpSpace,  *joint.cpConstraint, *unused)
  cpSpaceRemoveConstraint(*space, *joint);
  cpConstraintFree(*joint)               ;
EndProcedure

ProcedureC BreakableJointPostSolve(*joint.cpConstraint, *space.cpSpace)
  dt.cpFloat = cpSpaceGetCurrentTimeStep(*space);
  
  ;// Convert the impulse To a force by dividing it by the timestep.
  force.cpFloat = cpConstraintGetImpulse(*joint)/dt;
  maxForce.cpFloat = cpConstraintGetMaxForce(*joint)     ;
  
  ;// If the force is almost As big As the joint's max force, break it.
  If force > 0.9*maxForce
    cpSpaceAddPostStepCallback(*space, @BreakablejointPostStepRemove(), *joint, #Null);
  EndIf
EndProcedure

;- Debug Draw Functions
;- -----------------------------------------
ProcedureC DrawACircleShape(*_sprite.cpSprite, *_center.cpVect, _angle.cpFloat, _radius.cpFloat, _color.i)
  CompilerIf Subsystem("DX9")
    MP_Circle(*_center\x, *_center\y, _radius+1, _color)
    MP_LineXY(*_center\x, *_center\y, *_center\x + Cos(_angle) * _radius, *_center\y + Sin(_angle) * _radius, _color)  
  CompilerElse
    DrawingMode(#PB_2DDrawing_Outlined)
    Circle(*_center\x, *_center\y, _radius-1, _color)
    LineXY(*_center\x, *_center\y, *_center\x + Cos(_angle) * _radius, *_center\y + Sin(_angle) * _radius, _color)
    DrawingMode(#PB_2DDrawing_Default )
  CompilerEndIf
EndProcedure

ProcedureC DrawASegmentShape(*_sprite.cpSprite, *_pos1.cpVect, *_pos2.cpVect, _radius.cpFloat, _color.i)
  CompilerIf Subsystem("DX9")
    MP_LineXY(*_pos1\x, *_pos1\y, *_pos2\x, *_pos2\y, _color)
  CompilerElse
    If _radius > 0
      Circle(*_pos1\x-_radius, *_pos1\y - radius, _radius-1, _color)
      Circle(*_pos2\x - _radius, *_pos2\y-radius, _radius-1, _color)
      LineXY(*_pos1\x - _radius, *_pos1\y -_radius, *_pos2\x - _radius, *_pos2\y - _radius, _color)
      LineXY(*_pos1\x + _radius, *_pos1\y + _radius, *_pos2\x + _radius, *_pos2\y + _radius, _color)
    EndIf
    
    LineXY(*_pos1\x, *_pos1\y, *_pos2\x, *_pos2\y, _color)
  CompilerEndIf
EndProcedure


ProcedureC DrawAPolygonShape(*_sprite.cpSprite, _count.i, *_verts.cpVect, _color.i)
  *_first.cpVect = *_verts
  Static  *_verts1.cpVect
  
  For i=0 To _count -2
    *_verts1 = *_verts + SizeOf(cpVect)
    CompilerIf Subsystem("DX9")
      MP_LineXY(*_verts\x, *_verts\y, *_verts1\x, *_verts1\y, _color)
    CompilerElse
      LineXY(*_verts\x, *_verts\y, *_verts1\x, *_verts1\y, _color)
    CompilerEndIf  
    *_verts = *_verts1
  Next 
  CompilerIf Subsystem("DX9")
    MP_LineXY(*_verts\x, *_verts\y, *_first\x, *_first\y, _color)
  CompilerElse
    LineXY(*_verts\x, *_verts\y, *_first\x, *_first\y, _color)
  CompilerEndIf
EndProcedure

ProcedureC DrawASprite(*_sprite, *_center.cpVect, _angle.cpFloat)
  CompilerIf Subsystem("DX9")
    MP_RotateSprite(*_sprite, Degree(_angle))
    MP_DrawSprite(*_sprite, *_center\x-MP_SpriteGetWidth(*_sprite)/2, *_center\y-MP_SpriteGetHeight(*_sprite)/2)
  CompilerElse
    RotateSprite(*_sprite, Degree(_angle), #PB_Absolute)
    DisplayTransparentSprite(*_sprite, *_center\x-SpriteWidth(*_sprite)/2, *_center\y-SpriteHeight(*_sprite)/2)
  CompilerEndIf
EndProcedure

Procedure cpSetBorderLines()
  Static *shape.cpShape
  ; Create segments around the edge of the screen.
  *shape = cpSpaceAddShape(*space, cpSegmentShapeNew(*staticBody, cpv(_A,0,0), cpv(_B,0,wy-1), 0.0))
  cpShapeSetElasticity(*shape, 1.0)                                                               
  cpShapeSetFriction(*shape, 1.0)                                                                 
  cpShapeSetFilter(*shape, NOT_GRABBABLE_FILTER);
  
  *shape = cpSpaceAddShape(*space, cpSegmentShapeNew(*staticBody, cpv(_A,0,wy-1), cpv(_B,wx-1,wy-1), 0.0))
  cpShapeSetElasticity(*shape, 1.0)                                                                       
  cpShapeSetFriction(*shape, 1.0)                                                                         
  cpShapeSetFilter(*shape, NOT_GRABBABLE_FILTER);
  
  *shape = cpSpaceAddShape(*space, cpSegmentShapeNew(*staticBody, cpv(_A,wx-1,wy-1), cpv(_B,wx-1,0), 0.0));
  cpShapeSetElasticity(*shape, 1.0)                                                                       ;
  cpShapeSetFriction(*shape, 1.0)                                                                         ;
  cpShapeSetFilter(*shape, NOT_GRABBABLE_FILTER)                                                          ;
  
  *shape = cpSpaceAddShape(*space, cpSegmentShapeNew(*staticBody, cpv(_A,wx-1,0), cpv(_B,0,0), 0.0));
  cpShapeSetElasticity(*shape, 1.0)                                                                 ;
  cpShapeSetFriction(*shape, 1.0)   
  cpShapeSetFilter(*shape, NOT_GRABBABLE_FILTER)
EndProcedure

SetDrawCircleFunc(@DrawACircleShape())
SetDrawSegmentFunc(@DrawASegmentShape())
SetDrawPolygonFunc(@DrawAPolygonShape())
SetDrawSpriteFunc(@DrawASprite())



; IDE Options = PureBasic 5.70 LTS beta 1 (Windows - x86)
; CursorPosition = 40
; FirstLine = 9
; Folding = ---
; EnableXP
; HideErrorLog